/*
 * Copyright 1999 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

/*
 * Name:    d_sscalr.h
 *
 * Purpose: Header file for software emulated scaler.
 */

#ifndef __D_SSCALR_H__
#define __D_SSCALR_H__

typedef struct {
	MX_RECORD *motor_record;
	unsigned long num_datapoints;
	char datafile_name[ MXU_FILENAME_LENGTH + 1 ];
	double *motor_position;
	double *scaler_value;
} MX_SOFT_SCALER;

MX_API mx_status_type mxd_soft_scaler_initialize_type( long type );
MX_API mx_status_type mxd_soft_scaler_create_record_structures(
							MX_RECORD *record );
MX_API mx_status_type mxd_soft_scaler_finish_record_initialization(
							MX_RECORD *record );
MX_API mx_status_type mxd_soft_scaler_delete_record( MX_RECORD *record );
MX_API mx_status_type mxd_soft_scaler_read_parms_from_hardware(
							MX_RECORD *record );
MX_API mx_status_type mxd_soft_scaler_write_parms_to_hardware(
							MX_RECORD *record );
MX_API mx_status_type mxd_soft_scaler_open( MX_RECORD *record );
MX_API mx_status_type mxd_soft_scaler_close( MX_RECORD *record );

MX_API mx_status_type mxd_soft_scaler_clear( MX_SCALER *scaler );
MX_API mx_status_type mxd_soft_scaler_overflow_set( MX_SCALER *scaler );
MX_API mx_status_type mxd_soft_scaler_read( MX_SCALER *scaler );
MX_API mx_status_type mxd_soft_scaler_read_and_clear( MX_SCALER *scaler );
MX_API mx_status_type mxd_soft_scaler_is_busy( MX_SCALER *scaler );
MX_API mx_status_type mxd_soft_scaler_start( MX_SCALER *scaler );
MX_API mx_status_type mxd_soft_scaler_stop( MX_SCALER *scaler );
MX_API mx_status_type mxd_soft_scaler_get_mode( MX_SCALER *scaler );
MX_API mx_status_type mxd_soft_scaler_set_mode( MX_SCALER *scaler );

extern MX_RECORD_FUNCTION_LIST mxd_soft_scaler_record_function_list;
extern MX_SCALER_FUNCTION_LIST mxd_soft_scaler_scaler_function_list;

extern long mxd_soft_scaler_num_record_fields;
extern MX_RECORD_FIELD_DEFAULTS *mxd_soft_scaler_rfield_def_ptr;

#define MXD_SOFT_SCALER_STANDARD_FIELDS \
  {-1, -1, "motor_record", MXFT_RECORD, NULL, 0, {0}, \
	MXF_REC_TYPE_STRUCT, offsetof(MX_SOFT_SCALER, motor_record), \
	{0}, NULL, (MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY)}, \
  \
  {-1, -1, "num_datapoints", MXFT_ULONG, NULL, 0, {0}, \
	MXF_REC_TYPE_STRUCT, offsetof(MX_SOFT_SCALER, num_datapoints), \
	{0}, NULL, (MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY)}, \
  \
  {-1, -1, "datafile_name", MXFT_STRING, NULL, 1, {MXU_FILENAME_LENGTH}, \
	MXF_REC_TYPE_STRUCT, offsetof(MX_SOFT_SCALER, datafile_name), \
	{0}, NULL, (MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY)}

#endif /* __D_SSCALR_H__ */
