/*
 * Name:    i_epgpib.c
 *
 * Purpose: MX driver for the EPICS GPIB record written by Mark Rivers.
 *
 * Author:  William Lavender
 *
 * WARNING: I have not yet finished writing this driver.  It doesn't work yet!
 *
 *--------------------------------------------------------------------------
 *
 * Copyright 2000 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <stdio.h>

#include "mxconfig.h"

#if HAVE_EPICS

#include <stdlib.h>
#include <ctype.h>

#include "mx_util.h"
#include "mx_gpib.h"
#include "mx_rec.h"
#include "mx_drivr.h"
#include "i_epgpib.h"

/* The following are include files from the EPICS distributions. */

#include "tsDefs.h"
#include "cadef.h"
#include "ezca.h"

MX_RECORD_FUNCTION_LIST mxi_epics_gpib_record_function_list = {
	mxi_epics_gpib_initialize_type,
	mxi_epics_gpib_create_record_structures,
	mxi_epics_gpib_finish_record_initialization,
	mxi_epics_gpib_delete_record,
	mxi_epics_gpib_print_interface_structure,
	mxi_epics_gpib_read_parms_frm_hardware,
	mxi_epics_gpib_write_parms_to_hardware,
	mxi_epics_gpib_open,
	mxi_epics_gpib_close
};

MX_GPIB_FUNCTION_LIST mxi_epics_gpib_gpib_function_list = {
	mxi_epics_gpib_open_device,
	mxi_epics_gpib_close_device,
	mxi_epics_gpib_read_parms_frm_device,
	mxi_epics_gpib_write_parms_to_device,
	mxi_epics_gpib_read,
	mxi_epics_gpib_write,
	mxi_epics_gpib_interface_clear,
	mxi_epics_gpib_device_clear,
	mxi_epics_gpib_selective_device_clear,
	mxi_epics_gpib_local_lockout,
	mxi_epics_gpib_remote_enable,
	mxi_epics_gpib_go_to_local,
	mxi_epics_gpib_trigger,
	mxi_epics_gpib_wait_for_service_request,
	mxi_epics_gpib_serial_poll,
	mxi_epics_gpib_serial_poll_disable
};

MX_RECORD_FIELD_DEFAULTS mxi_epics_gpib_record_field_defaults[] = {
	MX_RECORD_STANDARD_FIELDS,
	MX_GPIB_STANDARD_FIELDS,
	MXI_EPICS_GPIB_STANDARD_FIELDS
};

long mxi_epics_gpib_num_record_fields
		= sizeof( mxi_epics_gpib_record_field_defaults )
			/ sizeof( mxi_epics_gpib_record_field_defaults[0] );

MX_RECORD_FIELD_DEFAULTS *mxi_epics_gpib_rfield_def_ptr
			= &mxi_epics_gpib_record_field_defaults[0];

#define EPICS_GPIB_DEBUG	FALSE

/* ---- */

static mx_status_type
mxi_epics_gpib_set_address( MX_EPICS_GPIB *epics_gpib,
				MX_GPIB *gpib, int address );
static mx_status_type
mxi_epics_gpib_set_transaction_mode( MX_EPICS_GPIB *epics_gpib,
					MX_GPIB *gpib,
					long mode );

static mx_status_type
mxi_epics_gpib_universal_command( MX_EPICS_GPIB *epics_gpib,
					MX_GPIB *gpib,
					long command );

static mx_status_type
mxi_epics_gpib_addressed_command( MX_EPICS_GPIB *epics_gpib,
					MX_GPIB *gpib,
					int address,
					long command );

/* ==== Private function for the driver's use only. ==== */

static mx_status_type
mxi_epics_gpib_get_pointers( MX_GPIB *gpib,
				MX_EPICS_GPIB **epics_gpib,
				const char *calling_fname )
{
	const char fname[] = "mxi_epics_gpib_get_pointers()";

	if ( gpib == (MX_GPIB *) NULL ) {
		return mx_error( MXE_NULL_ARGUMENT, fname,
			"The MX_GPIB pointer passed by '%s' was NULL.",
			calling_fname );
	}

	if ( epics_gpib == (MX_EPICS_GPIB **) NULL ) {
		return mx_error( MXE_NULL_ARGUMENT, fname,
			"The MX_EPICS_GPIB pointer passed by '%s' was NULL.",
			calling_fname );
	}

	*epics_gpib = (MX_EPICS_GPIB *) (gpib->record->record_type_struct);

	if ( *epics_gpib == (MX_EPICS_GPIB *) NULL ) {
		return mx_error( MXE_CORRUPT_DATA_STRUCTURE, fname,
		"The MX_EPICS_GPIB pointer for record '%s' is NULL.",
			gpib->record->name );
	}
	return MX_SUCCESSFUL_RESULT;
}

/* ==== Public functions ==== */

MX_EXPORT mx_status_type
mxi_epics_gpib_initialize_type( long type )
{
	/* Nothing needed here. */

	return MX_SUCCESSFUL_RESULT;
}

MX_EXPORT mx_status_type
mxi_epics_gpib_create_record_structures( MX_RECORD *record )
{
	const char fname[] = "mxi_epics_gpib_create_record_structures()";

	MX_GPIB *gpib;
	MX_EPICS_GPIB *epics_gpib;

	/* Allocate memory for the necessary structures. */

	gpib = (MX_GPIB *) malloc( sizeof(MX_GPIB) );

	if ( gpib == (MX_GPIB *) NULL ) {
		return mx_error( MXE_OUT_OF_MEMORY, fname,
		"Can't allocate memory for MX_GPIB structure." );
	}

	epics_gpib = (MX_EPICS_GPIB *) malloc( sizeof(MX_EPICS_GPIB) );

	if ( epics_gpib == (MX_EPICS_GPIB *) NULL ) {
		return mx_error( MXE_OUT_OF_MEMORY, fname,
		"Can't allocate memory for MX_EPICS_GPIB structure." );
	}

	/* Now set up the necessary pointers. */

	record->record_class_struct = gpib;
	record->record_type_struct = epics_gpib;
	record->class_specific_function_list
				= &mxi_epics_gpib_gpib_function_list;

	gpib->record = record;

	return MX_SUCCESSFUL_RESULT;
}

MX_EXPORT mx_status_type
mxi_epics_gpib_finish_record_initialization( MX_RECORD *record )
{
	return mx_gpib_finish_record_initialization( record );
}

MX_EXPORT mx_status_type
mxi_epics_gpib_delete_record( MX_RECORD *record )
{
        if ( record == NULL ) {
                return MX_SUCCESSFUL_RESULT;
        }
        if ( record->record_type_struct != NULL ) {
                free( record->record_type_struct );

                record->record_type_struct = NULL;
        }
        if ( record->record_class_struct != NULL ) {
                free( record->record_class_struct );

                record->record_class_struct = NULL;
        }
        return MX_SUCCESSFUL_RESULT;
}

MX_EXPORT mx_status_type
mxi_epics_gpib_print_interface_structure( FILE *file, MX_RECORD *record )
{
	MX_GPIB *gpib;
	MX_EPICS_GPIB *epics_gpib;
	char read_eos_char, write_eos_char;

	gpib = (MX_GPIB *) (record->record_class_struct);

	epics_gpib = (MX_EPICS_GPIB *) (record->record_type_struct);

	fprintf(file, "GPIB parameters for interface '%s':\n", record->name);
	fprintf(file, "  GPIB type              = EPICS_GPIB.\n\n");

	fprintf(file, "  name                   = %s\n", record->name);
	fprintf(file, "  EPICS record name      = %s\n",
					epics_gpib->epics_record_name);
	fprintf(file, "  default I/O timeout    = %d\n",
					gpib->default_io_timeout_value);
	fprintf(file, "  default EOI mode       = %d\n",
					gpib->default_eoi_mode);

	read_eos_char = (char) ( gpib->default_read_terminator & 0xff );

	if ( isprint(read_eos_char) ) {
		fprintf(file,
		      "  default read EOS char  = 0x%02x (%c)\n",
			read_eos_char, read_eos_char);
	} else {
		fprintf(file,
		      "  default read EOS char  = 0x%02x\n", read_eos_char);
	}

	write_eos_char = (char) ( gpib->default_write_terminator & 0xff );

	if ( isprint(write_eos_char) ) {
		fprintf(file,
		      "  default write EOS char = 0x%02x (%c)\n",
			write_eos_char, write_eos_char);
	} else {
		fprintf(file,
		      "  default write EOS char = 0x%02x\n", write_eos_char);
	}

	return MX_SUCCESSFUL_RESULT;
}

MX_EXPORT mx_status_type
mxi_epics_gpib_read_parms_frm_hardware( MX_RECORD *record )
{
	return MX_SUCCESSFUL_RESULT;
}

MX_EXPORT mx_status_type
mxi_epics_gpib_write_parms_to_hardware( MX_RECORD *record )
{
	return MX_SUCCESSFUL_RESULT;
}

MX_EXPORT mx_status_type
mxi_epics_gpib_open( MX_RECORD *record )
{
	const char fname[] = "mxi_epics_gpib_open()";

	MX_GPIB *gpib;
	MX_EPICS_GPIB *epics_gpib;
	char pvname[60];
	char *ezca_error_message;
	char mx_error_message[150];
	int ezca_status;
	long format, output_delimiter;
	mx_status_type mx_status;

	MX_DEBUG( 2,("%s invoked.", fname));

	if ( record == (MX_RECORD *) NULL ) {
		return mx_error( MXE_NULL_ARGUMENT, fname,
		"MX_RECORD pointer passed is NULL.");
	}

	gpib = (MX_GPIB *) (record->record_class_struct);

	mx_status = mxi_epics_gpib_get_pointers( gpib, &epics_gpib, fname );

	if ( mx_status.code != MXE_SUCCESS )
		return mx_status;

	/* Turn off the automatic printing of EZCA error messages
	 * to stderr, since we want to handle that ourselves anyway.
	 */

	ezcaAutoErrorMessageOff();

	/* Set the input format to binary. */

	strncpy( pvname, epics_gpib->epics_record_name, sizeof(pvname)-5 );
	strcat( pvname, ".IFMT" );

	format = MXF_EPICS_GPIB_BINARY_FORMAT;

	ezca_status = ezcaPut( pvname, ezcaLong, 1, &format );

	if ( ezca_status != EZCA_OK ) {
		ezcaGetErrorString( NULL, &ezca_error_message );
		sprintf( mx_error_message,
	"Error setting the EPICS GPIB input format for MX record '%s' "
	"to binary.  EZCA status = %d (%s)",
			record->name, ezca_status, ezca_error_message );
		ezcaFree( (void *) ezca_error_message );

		return mx_error(MXE_DEVICE_IO_ERROR, fname, mx_error_message);
	}

	/* Set the output format to binary. */

	strncpy( pvname, epics_gpib->epics_record_name, sizeof(pvname)-5 );
	strcat( pvname, ".OFMT" );

	format = MXF_EPICS_GPIB_BINARY_FORMAT;

	ezca_status = ezcaPut( pvname, ezcaLong, 1, &format );

	if ( ezca_status != EZCA_OK ) {
		ezcaGetErrorString( NULL, &ezca_error_message );
		sprintf( mx_error_message,
	"Error setting the EPICS GPIB output format for MX record '%s' "
	"to binary.  EZCA status = %d (%s)",
			record->name, ezca_status, ezca_error_message );
		ezcaFree( (void *) ezca_error_message );

		return mx_error(MXE_DEVICE_IO_ERROR, fname, mx_error_message);
	}

	/* Unconditionally turn off output delimiters. */

	strncpy( pvname, epics_gpib->epics_record_name, sizeof(pvname)-5 );
	strcat( pvname, ".ODEL" );

	output_delimiter = -1L;

	ezca_status = ezcaPut( pvname, ezcaLong, 1, &output_delimiter );

	if ( ezca_status != EZCA_OK ) {
		ezcaGetErrorString( NULL, &ezca_error_message );
		sprintf( mx_error_message,
	"Error setting the EPICS GPIB output format for MX record '%s' "
	"to binary.  EZCA status = %d (%s)",
			record->name, ezca_status, ezca_error_message );
		ezcaFree( (void *) ezca_error_message );

		return mx_error(MXE_DEVICE_IO_ERROR, fname, mx_error_message);
	}

	/* Find out how big the input buffer is for the EPICS record. */

	strncpy( pvname, epics_gpib->epics_record_name, sizeof(pvname)-5 );
	strcat( pvname, ".IMAX" );

	ezca_status = ezcaGet( pvname, ezcaLong, 1,
				&(epics_gpib->max_input_length) );

	if ( ezca_status != EZCA_OK ) {
		ezcaGetErrorString( NULL, &ezca_error_message );
		sprintf( mx_error_message,
	"Error getting EPICS input buffer length for MX record '%s'.  "
	"EZCA status = %d (%s)",
			record->name, ezca_status, ezca_error_message );
		ezcaFree( (void *) ezca_error_message );

		return mx_error(MXE_DEVICE_IO_ERROR, fname, mx_error_message);
	}

	/* Find out how big the output buffer is for the EPICS record. */

	strncpy( pvname, epics_gpib->epics_record_name, sizeof(pvname)-5 );
	strcat( pvname, ".OMAX" );

	ezca_status = ezcaGet( pvname, ezcaLong, 1,
				&(epics_gpib->max_output_length) );

	if ( ezca_status != EZCA_OK ) {
		ezcaGetErrorString( NULL, &ezca_error_message );
		sprintf( mx_error_message,
	"Error getting EPICS output buffer length for MX record '%s'.  "
	"EZCA status = %d (%s)",
			record->name, ezca_status, ezca_error_message );
		ezcaFree( (void *) ezca_error_message );

		return mx_error(MXE_DEVICE_IO_ERROR, fname, mx_error_message);
	}

	/* Set the EOS character. */

	if ( gpib->default_read_terminator > 0xff ) {
		return mx_error( MXE_ILLEGAL_ARGUMENT, fname,
		"The read terminator %#x is not allowed.  "
		"Only one character read terminators are allowed for "
		"the EPICS GPIB record.",
			gpib->default_read_terminator );
	}

	epics_gpib->current_eos_char = gpib->default_read_terminator;

	strncpy( pvname, epics_gpib->epics_record_name, sizeof(pvname)-5 );
	strcat( pvname, ".EOS" );

	ezca_status = ezcaPut( pvname, ezcaLong,
					1, &(epics_gpib->current_eos_char) );

	if ( ezca_status != EZCA_OK ) {
		ezcaGetErrorString( NULL, &ezca_error_message );
		sprintf( mx_error_message,
	"Error setting EPICS GPIB EOS character to %#lx "
	"for MX record '%s'.  EZCA status = %d (%s)",
			epics_gpib->current_eos_char, record->name,
			ezca_status, ezca_error_message );
		ezcaFree( (void *) ezca_error_message );

		return mx_error(MXE_DEVICE_IO_ERROR, fname, mx_error_message);
	}

	/* Set num_chars_to_read to -1.  This lets other routines know
	 * that we have not yet done our first read command.
	 */

	epics_gpib->num_chars_to_read = -1L;

	/* Do the same for num_chars_to_write. */

	epics_gpib->num_chars_to_write = -1L;

	/* Set the default timeout to 500 milliseconds.  This matches
	 * the default for the EPICS record.
	 */

	epics_gpib->default_timeout = 500L;

	/* Initialize the other variables to known values. */

	epics_gpib->current_address = -1L;
	epics_gpib->transaction_mode = -1L;
	epics_gpib->timeout = -1L;

	return MX_SUCCESSFUL_RESULT;
}

MX_EXPORT mx_status_type
mxi_epics_gpib_close( MX_RECORD *record )
{
	return MX_SUCCESSFUL_RESULT;
}

/* ========== Device specific calls ========== */

MX_EXPORT mx_status_type
mxi_epics_gpib_open_device( MX_GPIB *gpib, int address )
{
	return MX_SUCCESSFUL_RESULT;
}

MX_EXPORT mx_status_type
mxi_epics_gpib_close_device( MX_GPIB *gpib, int address )
{
	return MX_SUCCESSFUL_RESULT;
}

MX_EXPORT mx_status_type
mxi_epics_gpib_read_parms_frm_device( MX_GPIB *gpib, int address )
{
	return MX_SUCCESSFUL_RESULT;
}

MX_EXPORT mx_status_type
mxi_epics_gpib_write_parms_to_device( MX_GPIB *gpib, int address )
{
	return MX_SUCCESSFUL_RESULT;
}

MX_EXPORT mx_status_type
mxi_epics_gpib_read( MX_GPIB *gpib,
		int address,
		char *buffer,
		size_t max_bytes_to_read,
		size_t *bytes_read,
		int flags )
{
	const char fname[] = "mxi_epics_gpib_read()";

	MX_EPICS_GPIB *epics_gpib;
	char pvname[60];
	char *ezca_error_message;
	char mx_error_message[150];
	int ezca_status, first_time;
	long long_address, timeout, num_chars_read;
	mx_status_type mx_status;

	mx_status = mxi_epics_gpib_get_pointers( gpib, &epics_gpib, fname );

	if ( mx_status.code != MXE_SUCCESS )
		return mx_status;

	if ( max_bytes_to_read > epics_gpib->max_input_length ) {
		return mx_error( MXE_WOULD_EXCEED_LIMIT, fname,
	"Requested number of characters (%ld) for GPIB port '%s' is longer "
	"than the maximum input buffer length of %d.",
			max_bytes_to_read, gpib->record->name,
			epics_gpib->max_input_length );
	}

	/* Check to see if this is the first time we have tried to read
	 * from the EPICS GPIB port.
	 */

	if ( epics_gpib->num_chars_to_read == -1L ) {
		first_time = TRUE;
	} else {
		first_time = FALSE;
	}

	/* Change the transaction mode if needed. */

	if ( epics_gpib->transaction_mode != MXF_EPICS_GPIB_READ ) {

		mx_status = mxi_epics_gpib_set_transaction_mode( epics_gpib,
						gpib, MXF_EPICS_GPIB_READ );

		if ( mx_status.code != MXE_SUCCESS )
			return mx_status;
	}

	/* Change the EPICS timeout value if required. */

	if ( flags & MXF_GPIB_NOWAIT ) {
		timeout = 0L;
	} else {
		timeout = epics_gpib->default_timeout;
	}

	if ( epics_gpib->timeout != timeout ) {

		strncpy( pvname,
			epics_gpib->epics_record_name, sizeof(pvname)-5 );

		strcat( pvname, ".TMOT" );

		ezca_status = ezcaPut( pvname, ezcaLong, 1, &timeout );

		if ( ezca_status != EZCA_OK ) {
			ezcaGetErrorString( NULL, &ezca_error_message );
			sprintf( mx_error_message,
	"Error setting GPIB timeout to %ld milliseconds for MX record '%s'.  "
	"EZCA status = %d (%s)", timeout,
				gpib->record->name, ezca_status,
				ezca_error_message );
			ezcaFree( (void *) ezca_error_message );

			return mx_error(MXE_DEVICE_IO_ERROR, fname,
					mx_error_message);
		}
		epics_gpib->timeout = timeout;
	}

	/* Set the number of characters to read if required. */

	if ( epics_gpib->num_chars_to_read != max_bytes_to_read ) {

		strncpy( pvname,
			epics_gpib->epics_record_name, sizeof(pvname)-5 );

		strcat( pvname, ".NRRD" );

		ezca_status = ezcaPut(pvname, ezcaLong, 1, &max_bytes_to_read);

		if ( ezca_status != EZCA_OK ) {
			ezcaGetErrorString( NULL, &ezca_error_message );
			sprintf( mx_error_message,
	"Error setting GPIB number of characters to read to %ld characters "
	"for MX record '%s'.  EZCA status = %d (%s)",
				(long) max_bytes_to_read,
				gpib->record->name, ezca_status,
				ezca_error_message );
			ezcaFree( (void *) ezca_error_message );

			return mx_error(MXE_DEVICE_IO_ERROR, fname,
					mx_error_message);
		}
		epics_gpib->num_chars_to_read = max_bytes_to_read;
	}

	/* Change the EOS character if required. */

	if ( epics_gpib->current_eos_char != gpib->read_terminator[ address ] )
	{
		strncpy( pvname,
			epics_gpib->epics_record_name, sizeof(pvname)-5 );

		strcat( pvname, ".EOS" );

		ezca_status = ezcaPut( pvname, ezcaLong,
					1, &(gpib->read_terminator[address]) );

		if ( ezca_status != EZCA_OK ) {
			ezcaGetErrorString( NULL, &ezca_error_message );
			sprintf( mx_error_message,
			"Error setting EPICS GPIB EOS character to %#lx "
			"for MX record '%s'.  EZCA status = %d (%s)",
				epics_gpib->current_eos_char,
				gpib->record->name, ezca_status,
				ezca_error_message );
			ezcaFree( (void *) ezca_error_message );

			return mx_error(MXE_DEVICE_IO_ERROR, fname,
						mx_error_message);
		}

		epics_gpib->current_eos_char = gpib->read_terminator[address];
	}

	/* Change the GPIB address if required. */

	if ( epics_gpib->current_address != address ) {

		strncpy( pvname,
			epics_gpib->epics_record_name, sizeof(pvname)-5 );

		strcat( pvname, ".ADDR" );

		long_address = (long) address;

		ezca_status = ezcaPut( pvname, ezcaLong,
						1, &long_address );

		if ( ezca_status != EZCA_OK ) {
			ezcaGetErrorString( NULL, &ezca_error_message );
			sprintf( mx_error_message,
			"Error setting EPICS GPIB address to %ld "
			"for MX record '%s'.  EZCA status = %d (%s)",
				long_address, gpib->record->name,
				ezca_status, ezca_error_message );
			ezcaFree( (void *) ezca_error_message );

			return mx_error(MXE_DEVICE_IO_ERROR, fname,
						mx_error_message);
		}
	}

	/* Read characters into the buffer. */

	strncpy( pvname, epics_gpib->epics_record_name, sizeof(pvname)-5 );

	strcat( pvname, ".BINP" );

	ezca_status = ezcaGet( pvname, ezcaByte, max_bytes_to_read, buffer );

	if ( ezca_status != EZCA_OK ) {
		ezcaGetErrorString( NULL, &ezca_error_message );
		sprintf( mx_error_message,
		"Error reading characters for EPICS GPIB port '%s'.  "
		"EZCA status = %d (%s)",
			gpib->record->name, ezca_status,
			ezca_error_message );
		ezcaFree( (void *) ezca_error_message );

		return mx_error(MXE_DEVICE_IO_ERROR, fname, mx_error_message);
	}

	/* Find out how many characters were read. */

	strncpy( pvname, epics_gpib->epics_record_name, sizeof(pvname)-5 );

	strcat( pvname, ".NORD" );

	ezca_status = ezcaGet( pvname, ezcaByte, 1, &num_chars_read );

	if ( ezca_status != EZCA_OK ) {
		ezcaGetErrorString( NULL, &ezca_error_message );
		sprintf( mx_error_message,
		"Error finding out how many characters were read from "
		"EPICS GPIB port '%s'.  EZCA status = %d (%s)",
			gpib->record->name, ezca_status,
			ezca_error_message );
		ezcaFree( (void *) ezca_error_message );

		return mx_error(MXE_DEVICE_IO_ERROR, fname, mx_error_message);
	}

	*bytes_read = num_chars_read;

	return MX_SUCCESSFUL_RESULT;
}

MX_EXPORT mx_status_type
mxi_epics_gpib_write( MX_GPIB *gpib,
		int address,
		char *buffer,
		size_t bytes_to_write,
		size_t *bytes_written,
		int flags )
{
	const char fname[] = "mxi_epics_gpib_write()";

	MX_EPICS_GPIB *epics_gpib;
	char pvname[60];
	char *ezca_error_message;
	char mx_error_message[150];
	int ezca_status;
	long timeout;
	mx_status_type mx_status;

	mx_status = mxi_epics_gpib_get_pointers( gpib, &epics_gpib, fname );

	if ( mx_status.code != MXE_SUCCESS )
		return mx_status;

	MX_DEBUG(-2,("%s invoked.", fname));

	if ( bytes_to_write > epics_gpib->max_output_length ) {
		return mx_error( MXE_WOULD_EXCEED_LIMIT, fname,
	"Requested number of characters (%ld) for GPIB port '%s' is longer "
	"than the maximum output buffer length of %d.",
			bytes_to_write, gpib->record->name,
			epics_gpib->max_output_length );
	}

	/* Change the transaction mode if needed. */

	if ( epics_gpib->transaction_mode != MXF_EPICS_GPIB_WRITE ) {

		mx_status = mxi_epics_gpib_set_transaction_mode( epics_gpib,
						gpib, MXF_EPICS_GPIB_WRITE );

		if ( mx_status.code != MXE_SUCCESS )
			return mx_status;
	}

	/* Change the EPICS timeout value if required. */

	if ( flags & MXF_GPIB_NOWAIT ) {
		timeout = 0L;
	} else {
		timeout = epics_gpib->default_timeout;
	}

	if ( epics_gpib->timeout != timeout ) {

		strncpy( pvname,
			epics_gpib->epics_record_name, sizeof(pvname)-5 );

		strcat( pvname, ".TMOT" );

		ezca_status = ezcaPut( pvname, ezcaLong, 1, &timeout );

		if ( ezca_status != EZCA_OK ) {
			ezcaGetErrorString( NULL, &ezca_error_message );
			sprintf( mx_error_message,
	"Error setting GPIB timeout to %ld milliseconds for MX record '%s'.  "
	"EZCA status = %d (%s)", timeout,
				gpib->record->name, ezca_status,
				ezca_error_message );
			ezcaFree( (void *) ezca_error_message );

			return mx_error(MXE_DEVICE_IO_ERROR, fname,
					mx_error_message);
		}
		epics_gpib->timeout = timeout;
	}

	/* Set the number of characters to write if required. */

	if ( epics_gpib->num_chars_to_write != bytes_to_write ) {

		strncpy( pvname,
			epics_gpib->epics_record_name, sizeof(pvname)-5 );

		strcat( pvname, ".NOWT" );

		ezca_status = ezcaPut(pvname, ezcaLong, 1, &bytes_to_write);

		if ( ezca_status != EZCA_OK ) {
			ezcaGetErrorString( NULL, &ezca_error_message );
			sprintf( mx_error_message,
	"Error setting GPIB number of characters to write to %ld characters "
	"for MX record '%s'.  EZCA status = %d (%s)", (long) bytes_to_write,
				gpib->record->name, ezca_status,
				ezca_error_message );
			ezcaFree( (void *) ezca_error_message );

			return mx_error(MXE_DEVICE_IO_ERROR, fname,
					mx_error_message);
		}
		epics_gpib->num_chars_to_write = bytes_to_write;
	}

	/* Change the GPIB address if required. */

	if ( epics_gpib->current_address != address ) {

		mx_status = mxi_epics_gpib_set_address( epics_gpib,
							gpib, address );

		if ( mx_status.code != MXE_SUCCESS )
			return mx_status;
	}

	/* Write characters from the buffer. */

	strncpy( pvname, epics_gpib->epics_record_name, sizeof(pvname)-5 );

	strcat( pvname, ".BOUT" );

	ezca_status = ezcaPut( pvname, ezcaByte, bytes_to_write, buffer );

	if ( ezca_status != EZCA_OK ) {
		ezcaGetErrorString( NULL, &ezca_error_message );
		sprintf( mx_error_message,
		"Error reading characters for EPICS GPIB port '%s'.  "
		"EZCA status = %d (%s)",
			gpib->record->name, ezca_status,
			ezca_error_message );
		ezcaFree( (void *) ezca_error_message );

		return mx_error(MXE_DEVICE_IO_ERROR, fname, mx_error_message);
	}

	/* There seems to be no way to find out how many characters were
	 * actually written, so we leave the value of *num_chars alone.
	 */

	return MX_SUCCESSFUL_RESULT;
}

MX_EXPORT mx_status_type
mxi_epics_gpib_interface_clear( MX_GPIB *gpib )
{
	const char fname[] = "mxi_epics_gpib_interface_clear()";

	return mx_error( MXE_NOT_YET_IMPLEMENTED, fname,
	"Interface clear is not yet implemented for the EPICS GPIB record");
}

MX_EXPORT mx_status_type
mxi_epics_gpib_device_clear( MX_GPIB *gpib )
{
	const char fname[] = "mxi_epics_gpib_device_clear()";

	MX_EPICS_GPIB *epics_gpib;
	mx_status_type mx_status;

	mx_status = mxi_epics_gpib_get_pointers( gpib, &epics_gpib, fname );

	if ( mx_status.code != MXE_SUCCESS )
		return mx_status;

	mx_status = mxi_epics_gpib_universal_command(epics_gpib, gpib,
					MXF_EPICS_GPIB_DEVICE_CLEAR);

	return mx_status;
}

MX_EXPORT mx_status_type
mxi_epics_gpib_selective_device_clear( MX_GPIB *gpib, int address )
{
	const char fname[] = "mxi_epics_gpib_selective_device_clear()";

	MX_EPICS_GPIB *epics_gpib;
	mx_status_type mx_status;

	mx_status = mxi_epics_gpib_get_pointers( gpib, &epics_gpib, fname );

	if ( mx_status.code != MXE_SUCCESS )
		return mx_status;

	mx_status = mxi_epics_gpib_addressed_command(epics_gpib, gpib, address, 
					MXF_EPICS_GPIB_SELECTED_DEVICE_CLEAR);

	return mx_status;
}

MX_EXPORT mx_status_type
mxi_epics_gpib_local_lockout( MX_GPIB *gpib )
{
	const char fname[] = "mxi_epics_gpib_local_lockout()";

	MX_EPICS_GPIB *epics_gpib;
	mx_status_type mx_status;

	mx_status = mxi_epics_gpib_get_pointers( gpib, &epics_gpib, fname );

	if ( mx_status.code != MXE_SUCCESS )
		return mx_status;

	mx_status = mxi_epics_gpib_universal_command(epics_gpib, gpib,
					MXF_EPICS_GPIB_LOCAL_LOCKOUT);

	return mx_status;
}

MX_EXPORT mx_status_type
mxi_epics_gpib_remote_enable( MX_GPIB *gpib, int address )
{
	const char fname[] = "mxi_epics_gpib_remote_enable()";

	return mx_error( MXE_NOT_YET_IMPLEMENTED, fname,
	"Remote enable is not yet implemented for the EPICS GPIB record");
}

MX_EXPORT mx_status_type
mxi_epics_gpib_go_to_local( MX_GPIB *gpib, int address )
{
	const char fname[] = "mxi_epics_gpib_go_to_local()";

	MX_EPICS_GPIB *epics_gpib;
	mx_status_type mx_status;

	mx_status = mxi_epics_gpib_get_pointers( gpib, &epics_gpib, fname );

	if ( mx_status.code != MXE_SUCCESS )
		return mx_status;

	mx_status = mxi_epics_gpib_addressed_command(epics_gpib, gpib, address, 
					MXF_EPICS_GPIB_GO_TO_LOCAL);

	return mx_status;
}

MX_EXPORT mx_status_type
mxi_epics_gpib_trigger( MX_GPIB *gpib, int address )
{
	const char fname[] = "mxi_epics_gpib_trigger_device()";

	MX_EPICS_GPIB *epics_gpib;
	mx_status_type mx_status;

	MX_DEBUG( 2, ("%s invoked.", fname));

	mx_status = mxi_epics_gpib_get_pointers( gpib, &epics_gpib, fname );

	if ( mx_status.code != MXE_SUCCESS )
		return mx_status;

	mx_status = mxi_epics_gpib_addressed_command(epics_gpib, gpib, address, 
					MXF_EPICS_GPIB_GROUP_EXECUTE_TRIGGER);

	return mx_status;
}

MX_EXPORT mx_status_type
mxi_epics_gpib_wait_for_service_request( MX_GPIB *gpib, double timeout )
{
	const char fname[] = "mxi_epics_gpib_wait_for_service_request()";

	return mx_error( MXE_NOT_YET_IMPLEMENTED, fname,
		"Wait for SRQ is not yet implemented." );
}

MX_EXPORT mx_status_type
mxi_epics_gpib_serial_poll( MX_GPIB *gpib, int address,
				unsigned char *serial_poll_byte)
{
	const char fname[] = "mxi_epics_gpib_serial_poll()";

	MX_EPICS_GPIB *epics_gpib;
	char buffer[40];
	size_t bytes_read;
	mx_status_type mx_status;

	MX_DEBUG( 2, ("%s invoked.", fname));

	mx_status = mxi_epics_gpib_get_pointers( gpib, &epics_gpib, fname );

	if ( mx_status.code != MXE_SUCCESS )
		return mx_status;

	mx_status = mxi_epics_gpib_universal_command(epics_gpib, gpib,
					MXF_EPICS_GPIB_SERIAL_POLL_ENABLE);

	/* Get the serial poll byte. */

	mx_status = mxi_epics_gpib_read( gpib, address,
					buffer, sizeof(buffer)-1,
					&bytes_read, 0 );

	if ( mx_status.code != MXE_SUCCESS )
		return mx_status;

	*serial_poll_byte = (unsigned char) buffer[0];

	return mx_status;
}

MX_EXPORT mx_status_type
mxi_epics_gpib_serial_poll_disable( MX_GPIB *gpib )
{
	const char fname[] = "mxi_epics_gpib_serial_poll_disable()";

	MX_EPICS_GPIB *epics_gpib;
	mx_status_type mx_status;

	mx_status = mxi_epics_gpib_get_pointers( gpib, &epics_gpib, fname );

	if ( mx_status.code != MXE_SUCCESS )
		return mx_status;

	mx_status = mxi_epics_gpib_universal_command(epics_gpib, gpib,
					MXF_EPICS_GPIB_SERIAL_POLL_DISABLE);

	return mx_status;
}

/* ==== private functions ==== */

static mx_status_type
mxi_epics_gpib_set_address( MX_EPICS_GPIB *epics_gpib,
				MX_GPIB *gpib, int address )
{
	const char fname[] = "mxi_epics_gpib_set_address()";

	char pvname[60];
	char *ezca_error_message;
	char mx_error_message[150];
	long long_address;
	int ezca_status;

	strncpy( pvname, epics_gpib->epics_record_name, sizeof(pvname)-5 );

	strcat( pvname, ".ADDR" );

	long_address = (long) address;

	ezca_status = ezcaPut( pvname, ezcaLong, 1, &long_address );

	if ( ezca_status != EZCA_OK ) {
		ezcaGetErrorString( NULL, &ezca_error_message );
		sprintf( mx_error_message,
		"Error setting EPICS GPIB address to %ld "
		"for MX record '%s'.  EZCA status = %d (%s)",
			long_address, gpib->record->name,
			ezca_status, ezca_error_message );
		ezcaFree( (void *) ezca_error_message );

		return mx_error(MXE_DEVICE_IO_ERROR, fname,
					mx_error_message);
	}

	return MX_SUCCESSFUL_RESULT;
}

static mx_status_type
mxi_epics_gpib_set_transaction_mode( MX_EPICS_GPIB *epics_gpib,
					MX_GPIB *gpib,
					long mode )
{
	const char fname[] = "mxi_epics_gpib_set_transaction_mode()";

	char pvname[60];
	char *ezca_error_message;
	char mx_error_message[150];
	int ezca_status;

	strncpy( pvname, epics_gpib->epics_record_name, sizeof(pvname)-5 );
	strcat( pvname, ".TMOD" );

	ezca_status = ezcaPut( pvname, ezcaLong, 1,
				&(epics_gpib->transaction_mode) );

	if ( ezca_status != EZCA_OK ) {
		ezcaGetErrorString( NULL, &ezca_error_message );
		sprintf( mx_error_message,
	"Error setting GPIB transaction mode to %ld for MX record '%s'.  "
	"EZCA mx_status = %d (%s)", epics_gpib->transaction_mode,
			gpib->record->name, ezca_status,
			ezca_error_message );
		ezcaFree( (void *) ezca_error_message );

		return mx_error(MXE_DEVICE_IO_ERROR, fname, mx_error_message);
	}

	epics_gpib->transaction_mode = mode;

	return MX_SUCCESSFUL_RESULT;
}

static mx_status_type
mxi_epics_gpib_universal_command( MX_EPICS_GPIB *epics_gpib,
					MX_GPIB *gpib,
					long command )
{
	return MX_SUCCESSFUL_RESULT;
}

static mx_status_type
mxi_epics_gpib_addressed_command( MX_EPICS_GPIB *epics_gpib,
					MX_GPIB *gpib,
					int address,
					long command )
{
	mx_status_type mx_status;

	mx_status = mxi_epics_gpib_set_address( epics_gpib, gpib, address );

	if ( mx_status.code != MXE_SUCCESS )
		return mx_status;

	return MX_SUCCESSFUL_RESULT;
}

#endif /* HAVE_EPICS */
