/*
 * Name:    mx_mcs.h
 *
 * Purpose: MX system header file for multichannel scalers.
 *
 * Author:  William Lavender
 *
 *-------------------------------------------------------------------------
 *
 * Copyright 1999-2000 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#ifndef __MX_MCS_H__
#define __MX_MCS_H__

#include "mx_rec.h"

typedef struct {
	MX_RECORD *record; /* Pointer to the MX_RECORD structure that points
			    * to this MCS.
			    */

	int busy;
	int mode;
	int parameter_type;

	long max_num_scalers;
	long max_num_measurements;

	MX_RECORD *timer_record;
	MX_RECORD **scaler_record_array;

	unsigned long **data_array;

	double measurement_time;
	unsigned long num_scalers;
	unsigned long num_measurements;

	int readout_preference;

	long scaler_index;
	long measurement_index;

	long first_scaler_number;
} MX_MCS;

#define MXF_MCS_PREFER_READ_SCALER	1
#define MXF_MCS_PREFER_READ_MEASUREMENT	2
#define MXF_MCS_PREFER_READ_ALL		3

#define MXLV_MCS_MAX_NUM_SCALERS	1001
#define MXLV_MCS_MAX_NUM_MEASUREMENTS	1002
#define MXLV_MCS_DATA_ARRAY		1003
#define MXLV_MCS_MODE			1004
#define MXLV_MCS_MEASUREMENT_TIME	1005
#define MXLV_MCS_NUM_MEASUREMENTS	1006

#define MX_MCS_STANDARD_FIELDS \
  {MXLV_MCS_MAX_NUM_SCALERS, -1, "max_num_scalers", MXFT_LONG, NULL, 0, {0}, \
	MXF_REC_CLASS_STRUCT, offsetof(MX_MCS, max_num_scalers), \
	{0}, NULL, (MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY)}, \
  \
  {MXLV_MCS_MAX_NUM_MEASUREMENTS, -1, "max_num_measurements", \
					MXFT_LONG, NULL, 0, {0}, \
	MXF_REC_CLASS_STRUCT, offsetof(MX_MCS, max_num_measurements), \
	{0}, NULL, (MXFF_IN_DESCRIPTION | MXFF_IN_SUMMARY)}, \
  \
  {MXLV_MCS_DATA_ARRAY, -1, "data_array", \
		MXFT_ULONG, NULL, 2, {MXU_VARARGS_LENGTH, MXU_VARARGS_LENGTH},\
	MXF_REC_CLASS_STRUCT, offsetof(MX_MCS, data_array), \
	{sizeof(unsigned long), sizeof(unsigned long *)}, NULL, MXFF_VARARGS}

typedef struct {
	mx_status_type ( *start ) ( MX_MCS *mcs );
	mx_status_type ( *stop ) ( MX_MCS *mcs );
	mx_status_type ( *clear ) ( MX_MCS *mcs );
	mx_status_type ( *busy ) ( MX_MCS *mcs );
	mx_status_type ( *read_all ) ( MX_MCS *mcs );
	mx_status_type ( *read_scaler ) ( MX_MCS *mcs );
	mx_status_type ( *read_measurement ) ( MX_MCS *mcs );
	mx_status_type ( *get_parameter ) ( MX_MCS *mcs );
	mx_status_type ( *set_parameter ) ( MX_MCS *mcs );
} MX_MCS_FUNCTION_LIST;

MX_API mx_status_type mx_mcs_get_pointers( MX_RECORD *mcs_record,
	MX_MCS **mcs, MX_MCS_FUNCTION_LIST **function_list_ptr,
	const char *calling_fname );

MX_API mx_status_type mx_mcs_finish_record_initialization(
					MX_RECORD *mcs_record );

MX_API mx_status_type mx_mcs_start( MX_RECORD *mcs_record );
MX_API mx_status_type mx_mcs_stop( MX_RECORD *mcs_record );
MX_API mx_status_type mx_mcs_clear( MX_RECORD *mcs_record );
MX_API mx_status_type mx_mcs_is_busy( MX_RECORD *mcs_record, int *busy );

MX_API mx_status_type mx_mcs_read_all( MX_RECORD *mcs_record );

MX_API mx_status_type mx_mcs_read_scaler( MX_RECORD *mcs_record,
					unsigned long scaler_index );

MX_API mx_status_type mx_mcs_read_measurement( MX_RECORD *mcs_record,
					unsigned long measurement_index );

MX_API mx_status_type mx_mcs_get_mode( MX_RECORD *mcs_record, int *mode );
MX_API mx_status_type mx_mcs_set_mode( MX_RECORD *mcs_record, int mode );

MX_API mx_status_type mx_mcs_get_measurement_time( MX_RECORD *mcs_record,
					double *measurement_time );
MX_API mx_status_type mx_mcs_set_measurement_time( MX_RECORD *mcs_record,
					double measurement_time );

MX_API mx_status_type mx_mcs_get_num_measurements( MX_RECORD *mcs_record,
					unsigned long *num_measurements );
MX_API mx_status_type mx_mcs_set_num_measurements( MX_RECORD *mcs_record,
					unsigned long num_measurements );

#endif /* __MX_MCS_H__ */
