/*
 * Name:     mscc.c
 *
 * Purpose:  This program is a front end to the Microsoft C 6.0 "cl"
 *           command.  It's purpose is to filter out warning messages
 *           that we don't want to see.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define MESSAGE_FILENAME	"_temp_.msc"

#define ERROR_CODE_LENGTH	16

#define FALSE	0
#define TRUE	1

main( int argc, char *argv[] )
{
	static char ignored_message[][ERROR_CODE_LENGTH + 1] = {

	": warning C4011:",  /* identifier truncated to ... */
	": warning C4100:",  /* unreferenced formal parameter */
	": warning C4127:",  /* conditional expression is constant */

	};

	static int num_ignored_messages 
		= sizeof( ignored_message ) / ERROR_CODE_LENGTH;

	char buffer[500];
	char *ptr;
	int i, j, ignore, length, status;
	FILE *message_file;

	/* Generate the command line. */

	strcpy( buffer, "cl -nologo -AL -G0 -W4" );

	for ( i = 1; i < argc; i++ ) {
		strcat( buffer, " " );
		strcat( buffer, argv[i] );
	}

	strcat( buffer, " > _temp_.msc" );

	length = strlen( buffer );

	if ( length > 127 ) {
		fprintf( stderr,
	"mscc: Internally generated command line is too long. (length = %d)\n",
			length );
		fprintf( stderr,
	"mscc: Generated command line = '%s'\n", buffer );
		exit(1);
	}

	/* Now execute the command line. */

	status = system( buffer );

	if ( status != 0 ) {
		fprintf( stderr,
"mscc: Error code %d occurred while executing the command line '%s'.\n",
		buffer );

		exit( status );
	}

	/* Read the text generated by this command. */

	message_file = fopen( MESSAGE_FILENAME, "r" );

	if ( message_file == NULL ) {
		fprintf( stderr,
"mscc: Unable to read the compiler output message file '_temp_.msc'\n");
		exit(1);
	}

	i = 0;

	fgets( buffer, sizeof buffer, message_file );

	while ( !feof( message_file ) ) {

		/* Is this an error message?  If not, then pass it through. */

		ptr = strchr( buffer, ':' );

		ignore = FALSE;

		if ( ptr != NULL ) {

			for ( j = 0; j < num_ignored_messages; j++ ) {

				if (strncmp( ptr, ignored_message[j],
					ERROR_CODE_LENGTH ) == 0) {

					ignore = TRUE;

					break;	/* exit the for() loop. */
				}
			}
		}

		if ( ignore == FALSE ) {
			fprintf( stdout, "%s", buffer );

			i++;
		}

		fgets( buffer, sizeof buffer, message_file );
	}

	fclose( message_file );

	/* Delete the message file */

	sprintf( buffer, "erase %s", MESSAGE_FILENAME );

	(void) system( buffer );

	if ( i > 1 ) {
		exit(1);
	} else if ( i < 0 ) {
		exit(2);
	} else {
		exit(0);
	}

	return 1;
}
